@@ -1,32 +1,32 @@
 /* This file contains a function R_phyclust_*_step() called by R wraps
-   phyclust.*.step() in "R/f_phyclust_em_step.r", and this function calls
-   the relative functions in "src/phyclust/" and "src/".
-
-   Writen: Wei-Chen Chen on 2009/11/12.
-*/
+ * phyclust.*.step() in "R/f_phyclust_em_step.r", and this function calls
+ * the relative functions in "src/phyclust/" and "src/".
+ *
+ * Writen: Wei-Chen Chen on 2009/11/12. */
 
 #include "R_phyclust.h"
 
 /* This function calls functions in "src/phyclust/phyclust_em_step.c"
-   and is called by phyclust.em.step() using .Call()
-   in "R/f_phyclust_em_step.r".
-   Input:
-     R_N_X_org: SEXP[1], number of sequences.
-     R_L: SEXP[1], length of sequences.
-     R_X: SEXP[1], sequences.
-     R_K: SEXP[1], number of clusters.
-     R_Eta: SEXP[1], Eta.
-     R_Mu: SEXP[1], Mu.
-     R_vect: SEXP[1], vect contains pi, kappa, and Tt.
-     R_substitution_model: SEXP[1], substitution model.
-     R_identifier: SEXP[1], identifier.
-     R_code_type: SEXP[1], code_type.
-   Output:
-     ret: a list contains everythings returned from phyclust in C.
-*/
+ * and is called by phyclust.em.step() using .Call()
+ * in "R/f_phyclust_em_step.r".
+ * Input:
+ *   R_N_X_org: SEXP[1], number of sequences.
+ *   R_L: SEXP[1], length of sequences.
+ *   R_X: SEXP[1], sequences.
+ *   R_K: SEXP[1], number of clusters.
+ *   R_Eta: SEXP[1], Eta.
+ *   R_Mu: SEXP[1], Mu.
+ *   R_vect: SEXP[1], vect contains pi, kappa, and Tt.
+ *   R_substitution_model: SEXP[1], substitution model.
+ *   R_identifier: SEXP[1], identifier.
+ *   R_code_type: SEXP[1], code_type.
+ *   R_label: SEXP[1], labeles.
+ * Output:
+ *   ret: a list contains everythings returned from phyclust in C. */
 SEXP R_phyclust_em_step(SEXP R_N_X_org, SEXP R_L, SEXP R_X, SEXP R_K,
 		SEXP R_Eta, SEXP R_Mu, SEXP R_vect,
-		SEXP R_substitution_model, SEXP R_identifier, SEXP R_code_type){
+		SEXP R_substitution_model, SEXP R_identifier, SEXP R_code_type,
+		SEXP R_label){
 	/* Declare variables for calling C. */
 	int *C_N_X_org, *C_L, *C_K;
 	double *C_vect;
@@ -81,7 +81,14 @@ SEXP R_phyclust_em_step(SEXP R_N_X_org, SEXP R_L, SEXP R_X, SEXP R_K,
 		tmp_ptr_double++;
 	}
 	update_phyclust_struct(pcs);
+
+	/* Assign labels. */
+	R_update_phyclust_label(pcs, R_label);
+
+	/* Assign empcs. */
 	empcs = initialize_em_phyclust_struct(pcs);
+
+	/* Assign function pointers. */
 	EMFP = initialize_em_fp(EMC, pcs);
 
 	/* Assign QA. */
@@ -115,27 +122,27 @@ SEXP R_phyclust_em_step(SEXP R_N_X_org, SEXP R_L, SEXP R_X, SEXP R_K,
 
 
 /* This function calls E_step_simple() in "src/phyclust/phyclust_em_step.c"
-   and is called by phyclust.e.step() using .Call()
-   in "R/f_phyclust_em_step.r".
-   Input:
-     R_N_X_org: SEXP[1], number of sequences.
-     R_L: SEXP[1], length of sequences.
-     R_X: SEXP[1], sequences.
-     R_K: SEXP[1], number of clusters.
-     R_Eta: SEXP[1], Eta.
-     R_Mu: SEXP[1], Mu.
-     R_vect: SEXP[1], vect contains pi, kappa, and Tt.
-     R_substitution_model: SEXP[1], substitution model.
-     R_identifier: SEXP[1], identifier.
-     R_code_type: SEXP[1], code_type.
-     R_Z_state: SEXP[1], Z_state.
-   Output:
-     ret: Z_normalized.
-*/
+ * and is called by phyclust.e.step() using .Call()
+ * in "R/f_phyclust_em_step.r".
+ * Input:
+ *   R_N_X_org: SEXP[1], number of sequences.
+ *   R_L: SEXP[1], length of sequences.
+ *   R_X: SEXP[1], sequences.
+ *   R_K: SEXP[1], number of clusters.
+ *   R_Eta: SEXP[1], Eta.
+ *   R_Mu: SEXP[1], Mu.
+ *   R_vect: SEXP[1], vect contains pi, kappa, and Tt.
+ *   R_substitution_model: SEXP[1], substitution model.
+ *   R_identifier: SEXP[1], identifier.
+ *   R_code_type: SEXP[1], code_type.
+ *   R_Z_state: SEXP[1], Z_state.
+ *   R_label: SEXP[1], labels.
+ * Output:
+ *   ret: Z_normalized. */
 SEXP R_phyclust_e_step(SEXP R_N_X_org, SEXP R_L, SEXP R_X, SEXP R_K,
 		SEXP R_Eta, SEXP R_Mu, SEXP R_vect,
 		SEXP R_substitution_model, SEXP R_identifier, SEXP R_code_type,
-		SEXP R_Z_state){
+		SEXP R_Z_state, SEXP R_label){
 	/* Declare variables for calling C. */
 	int *C_N_X_org, *C_L, *C_K, *C_Z_state;
 	double *C_vect;
@@ -182,7 +189,14 @@ SEXP R_phyclust_e_step(SEXP R_N_X_org, SEXP R_L, SEXP R_X, SEXP R_K,
 	}
 	pcs->Eta = REAL(R_Eta);					/* Read only. */
 	update_phyclust_struct(pcs);
+
+	/* Assign labels. */
+	R_update_phyclust_label(pcs, R_label);
+
+	/* Assign empcs. */
 	empcs = initialize_em_phyclust_struct(pcs);
+
+	/* Assign function pointers. */
 	EMFP = initialize_em_fp(EMC, pcs);
 
 	/* Assign QA. */
@@ -231,12 +245,14 @@ SEXP R_phyclust_e_step(SEXP R_N_X_org, SEXP R_L, SEXP R_X, SEXP R_K,
      R_substitution_model: SEXP[1], substitution model.
      R_identifier: SEXP[1], identifier.
      R_code_type: SEXP[1], code_type.
+     R_label: SEXP[1], labels.
    Output:
      ret: a list contains everythings returned from phyclust in C.
 */
 SEXP R_phyclust_m_step(SEXP R_N_X_org, SEXP R_L, SEXP R_X, SEXP R_K,
 		SEXP R_vect, SEXP R_Z_normalized,
-		SEXP R_substitution_model, SEXP R_identifier, SEXP R_code_type){
+		SEXP R_substitution_model, SEXP R_identifier, SEXP R_code_type,
+		SEXP R_label){
 	/* Declare variables for calling C. */
 	int *C_N_X_org, *C_L, *C_K;
 	double *C_vect, *C_Z_normalized;
@@ -284,7 +300,14 @@ SEXP R_phyclust_m_step(SEXP R_N_X_org, SEXP R_L, SEXP R_X, SEXP R_K,
 	assign_class(pcs);
 	assign_Mu_by_class(pcs->N_X_org, pcs->K, pcs->L, pcs->ncode, pcs->missing_index, pcs->class_id, pcs->X_org, pcs->Mu);
 	update_phyclust_struct(pcs);
+
+	/* Assign labels. */
+	R_update_phyclust_label(pcs, R_label);
+
+	/* Assign empcs. */
 	empcs = initialize_em_phyclust_struct(pcs);
+
+	/* Assign function pointers. */
 	EMFP = initialize_em_fp(EMC, pcs);
 
 	/* Assign QA. */
