@@ -1,11 +1,13 @@
 /* This file contains declarations for phyclust. */
 
-#ifndef __PHYCLSUT_STRUCT_
-#define __PHYCLSUT_STRUCT_
+#ifndef __PHYCLUST_STRUCT_
+#define __PHYCLUST_STRUCT_
 
 #include "phyclust_qmatrix_array.h"
+#include "phyclust_label.h"
+
+typedef struct _phyclust_struct		phyclust_struct;
 
-typedef struct _phyclust_struct	phyclust_struct;
 
 /* Phyloclustering structure for storing data and results. */
 struct _phyclust_struct{
@@ -18,22 +20,19 @@ struct _phyclust_struct{
 	/* Storage. */
 	int		n_param;		/* Number of parameters, including Mu's, and eta's. */
 	int		N_X_org;		/* Number of original sequences. */
-	int		N_X_unique;		/* Number of unique sequnces, used for initialization only*/
 	int		N_X;			/* Number of sequences. */
 	int		N_seg_site;		/* Total segregating sites, including all are missing. */
 	int		L;			/* Number of loci. */
 	int		K;			/* Number of clusters. */
 	int		**X_org;		/* Original data pointer, dim = N_X_org * L. */
-	int		**X_unique;		/* Unique data pointer, dim = N_X_unique * L. */
 	int		**X;			/* Data pointer, dim = N_X * L. */
-	int		*map_X_org_to_X_unique;	/* Map indexes from X_org to X_unique, dim = N_X_org, used for initialization only. */
-	int		*map_X_unique_to_X_org;	/* Map indexes from X_unique to X_org, dim = N_X_unique, used for initialization only. */
-	int		*replication_X_unique;	/* Count replications of each unique sequence, dim = N_X_unique. */
 	int		*map_X_org_to_X;	/* Map indexes from X_org to X, dim = N_X_org. */
 	int		*map_X_to_X_org;	/* Map indexes from X to X_org, dim = N_X. */
 	int		*replication_X;		/* Count replications of each unique sequence, dim = N_X. */
 	/* Summarizd information for X. */
 	int		*seg_site_id;		/* Segregating site id, dim = N_seg_site. */
+	/* Label. */
+	phyclust_label	*label;			/* For labels. */
 
 /* Dynamical variables, used in copy functions only. */
 	/* Parameters. */
@@ -65,4 +64,4 @@ void print_X(phyclust_struct *pcs);
 void print_Mu(phyclust_struct *pcs);
 void print_class_id(phyclust_struct *pcs);
 
-#endif	/* End of __PHYCLSUT_STRUCT_. */
+#endif	/* End of __PHYCLUST_STRUCT_. */
