@@ -271,14 +271,15 @@ int cmatinv( complex x[], int n, int m, double space[])
 */
    int i,j,k, *irow=(int*) space;
    double xmaxsize, ee=1e-20;
-   complex xmax, t,t1;
+//WCC   complex xmax, t,t1;
+   complex t,t1;
 
    FOR(i,n)  {
        xmaxsize = 0.;
        for (j=i; j<n; j++) {
           if ( xmaxsize < csize (x[j*m+i]))  {
                xmaxsize = csize (x[j*m+i]);
-               xmax = x[j*m+i];
+//WCC               xmax = x[j*m+i];
                irow[i] = j;
           }
        }
